{% extends "admin/change_form.html" %}
{% load i18n %}

{% block content %}
    <h1>{% translate "Export form submissions" %}</h1>
    {% if errors %}
        <p class="errornote">
            {% blocktranslate count errors|length as counter %}Please correct the error below.{% plural %}Please correct the errors below.{% endblocktranslate %}
        </p>
        {{ adminform.non_field_errors }}
    {% endif %}
    <form action="." method="post">
        {% csrf_token %}
        {% block aldryn_form_hidden %}{% endblock %}
        <div>
            <fieldset class="module aligned">
                {% for field in adminform %}
                    <div class="form-row">
                        <div{% if field.errors %} class="errors"{% endif %}>
                            {% if field.errors %}{{ field.errors }}{% endif %}
                            {{ field.label_tag }}
                            {{ field }}
                        </div>
                    </div>
                {% endfor %}
            </fieldset>
        </div>
        <div class="submit-row">
            {% block aldryn_form_buttons %}
                <input type="submit" value="{% translate "Export" %}" class="default" name="export">
            {% endblock %}
        </div>
    </form>
{% endblock %}
