(()=>{const e=e=>{for(const t of e.getElementsByClassName("more"))t.classList.toggle("d-none")},t=()=>{const e=document.createElement("span"),t=document.createElement("img");return t.src="/static/admin/img/icon-viewlink.svg",t.title="Toggle more/less.",e.appendChild(t),e.style.cursor="pointer",e.classList.add("item-after"),e};document.addEventListener("DOMContentLoaded",(()=>{for(const n of document.getElementsByClassName("aldryn-forms-data")){const s=t();n.prepend(s),s.addEventListener("click",(t=>{e(t.target.closest(".aldryn-forms-data"))}))}const n=document.querySelector("#result_list th.column-display_data div.text");if(n){const s=t();s.addEventListener("click",(()=>{for(const t of document.getElementsByClassName("aldryn-forms-data"))e(t)})),n.appendChild(s)}}))})();
//# sourceMappingURL=admin_list.js.map