# Generated by Django 5.2.6 on 2025-11-12 09:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("aldryn_forms", "0025_alter_formplugin_action_backend_and_language"),
    ]

    operations = [
        migrations.AddField(
            model_name="fileuploadfieldplugin",
            name="is_private",
            field=models.BooleanField(
                default=False, help_text="The file is private.", verbose_name="Private"
            ),
        ),
        migrations.AddField(
            model_name="imageuploadfieldplugin",
            name="is_private",
            field=models.BooleanField(
                default=False, help_text="The file is private.", verbose_name="Private"
            ),
        ),
        migrations.AddField(
            model_name="multiplefilesuploadfieldplugin",
            name="is_private",
            field=models.BooleanField(
                default=False, help_text="The files are private.", verbose_name="Private"
            ),
        ),
    ]
