# Generated by Django 4.2.20 on 2025-03-19 12:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0019_submittedtobesent_formsubmission_post_ident_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='Webhook',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True, verbose_name='Name')),
                ('url', models.URLField(unique=True, verbose_name='Webhook URL')),
                ('method', models.CharField(choices=[('post', 'POST'), ('json', 'JSON')], default=('post', 'POST'), max_length=20, verbose_name='Data transfer method')),
                ('transform', models.JSONField(blank=True, null=True, verbose_name='Data transform')),
            ],
        ),
        migrations.AddField(
            model_name='formplugin',
            name='webhooks',
            field=models.ManyToManyField(blank=True, to='aldryn_forms.webhook'),
        ),
        migrations.AddField(
            model_name='formsubmission',
            name='webhooks',
            field=models.ManyToManyField(blank=True, to='aldryn_forms.webhook'),
        ),
        migrations.AddField(
            model_name='submittedtobesent',
            name='webhooks',
            field=models.ManyToManyField(blank=True, to='aldryn_forms.webhook'),
        ),
    ]
