{% extends "admin/change_list.html" %}
{% load i18n static admin_urls %}

{% block extrastyle %}
  {{ block.super }}
  <link rel="stylesheet" href="{% static "aldryn_forms/css/admin-list.css" %}">
{% endblock %}

{% block extrahead %}
    {{ block.super }}
    <script src="{% static "aldryn_forms/js/admin_list.js" %}" defer></script>
{% endblock %}

{% block result_list %}
    <div class="aldryn-forms-formsubmission">
        {{ block.super }}
    </div>
{% endblock %}

{% block object-tools %}
	<ul class="object-tools">
        {% block object-tools-items %}
            <li>
                {% url cl.opts|admin_urlname:'export' as export_url %}
                <a href="{{ export_url }}" class="addlink">{% translate "Export Form Entries" %}</a>
            </li>
            {% if has_add_permission %}
                {{ block.super }}
            {% endif %}
        {% endblock %}
    </ul>
{% endblock %}
