{% load i18n aldryn_forms_admin_tags %}
<table class="submitted-data">
<thead>
    <tr>
        <th>{% translate "Label" %}</th>
        <th>{% translate "Field" %}</th>
        <th>{% translate "Value" %}</th>
    </tr>
</thead>
<tbody>
    {% for field in data %}
        <tr>
            <td>{{ field.label }}</td>
            <td>{{ field.name }}</td>
            <td>{{ field.value|media_filer_public_link }}</td>
        </tr>
    {% endfor %}
</tbody>
</table>
